/**
 * Test case for setup.
 * Runs with mocha.
 */
'use strict'

const setup = require('../lib/setup.js')
const assert = require('assert')
const co = require('co')
const coz = require('coz')

describe('setup', () => {
  before(() => co(function * () {
  }))

  after(() => co(function * () {

  }))

  it('Render views', () => co(function * () {
    let buds = setup({
      name: [
        'SplashTop',
        'AccountDetail'
      ],
      type: 'view'
    })
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/views/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render fragments', () => co(function * () {
    let buds = setup({
      name: [
        'foo/SampleFragment',
        'bar/sample_baz_baz'
      ],
      type: 'fragment'
    })
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/fragments/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render pages', () => co(function * () {
    let buds = setup.page([
      'sample_page_bar'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/pages/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render mixins', () => co(function * () {
    let buds = setup({
      name: [
        'bar/sample_mixin_bar'
      ],
      type: 'mixin'
    })
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/mixins/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render mixin groups', () => co(function * () {
    let buds = setup.mixinGroup([
      'bar/Viewable'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/mixins/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render services', () => co(function * () {
    let buds = setup.service([
      'bar/sample_service_bar'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/services/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render stores', () => co(function * () {
    let buds = setup.store([
      'bar/sample_store_bar'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/stores/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render store accesses', () => co(function * () {
    let buds = setup.storeAccess([
      'dispatch'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/stores/accessing/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render store reduces', () => co(function * () {
    let buds = setup.storeReduce([
      'listing'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/stores/reducing/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render themes', () => co(function * () {
    let buds = setup({
      name: [
        'foo/SampleTheme',
        'bar/sample_themes'
      ],
      type: 'fragment'
    })
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/themes/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render browser', () => co(function * () {
    let buds = setup.browser([
      'Index'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/browser/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render browser render page', () => co(function * () {
    let buds = setup.browser([
      'render-page'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/browser/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render fragments index', () => co(function * () {
    let bud = setup.index(`${__dirname}/../lib`)
    assert.ok(bud)
    assert.ok(bud.tmpl)
    bud.path = `${__dirname}/../tmp/fragments/${bud.path}`
    bud.mkdirp = true
    bud.force = true
    yield coz.render(bud)
  }))

  it('Render pre defined fragments', () => co(function * () {
    let buds = setup.fragment([
      'footer',
      'header',
      'main',
      'theme'
    ])
    for (let bud of buds) {
      assert.ok(buds)
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/fragments/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render views index', () => co(function * () {
    let bud = setup.index(`${__dirname}/../lib`)
    assert.ok(bud)
    assert.ok(bud.tmpl)
    bud.path = `${__dirname}/../tmp/views/${bud.path}`
    bud.mkdirp = true
    bud.force = true
    yield coz.render(bud)
  }))

  it('Render theme', () => co(function * () {
    let buds = setup.fragment('Theme')
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/theme/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))

  it('Render helper', () => co(function * () {
    let buds = setup.helper([
      'foo'
    ])
    assert.ok(buds)
    for (let bud of buds) {
      assert.ok(bud.tmpl)
      bud.path = `${__dirname}/../tmp/helpers/${bud.path}`
      bud.mkdirp = true
      bud.force = true
      yield coz.render(bud)
    }
  }))
})

/* global describe, before, after, it */

